@@ -18,7 +18,7 @@ class Receiver(HawkAuthority):
                  url,
                  method,
                  content='',
-                 content_type='text/plain',
+                 content_type='',
                  seen_nonce=None,
                  localtime_offset_in_seconds=0,
                  timestamp_skew_in_seconds=default_ts_skew_in_seconds,
@@ -66,7 +66,7 @@ class Receiver(HawkAuthority):
 
     def respond(self,
                 content='',
-                content_type='text/plain',
+                content_type='',
                 ext=None):
 
         log.debug('generating response header')
@@ -76,12 +76,14 @@ class Receiver(HawkAuthority):
                             ext=ext,
                             app=self.parsed_header.get('app', None),
                             dlg=self.parsed_header.get('dlg', None),
-                            nonce=self.parsed_header['nonce'],
                             method=self.resource.method,
                             content=content,
-                            content_type=content_type)
+                            content_type=content_type,
+                            nonce=self.parsed_header['nonce'],
+                            timestamp=self.parsed_header['ts'])
 
         mac = calculate_mac('response', resource)
 
-        self.response_header = self._make_header(resource, mac)
+        self.response_header = self._make_header(resource, mac,
+                                                 additional_keys=['ext'])
         return self.response_header
